(($, Drupal, window) => {
  Drupal.behaviors.viewsFiltersSummary = {
    attach(context) {
      /**
       * Reset all exposed form inputs to an empty value/default state.
       * @param {jQuery} $exposedForm
       *   The exposed form to reset.
       */
      function reset($exposedForm) {
        $exposedForm.find(':input').each(function resetInput() {
          switch (this.type) {
            case 'password':
            case 'select-multiple':
            case 'select-one':
            case 'text':
            case 'textarea':
            case 'date':
              this.value = '';
              break;
            case 'checkbox':
            case 'radio':
              this.checked = false;
              break;
            default:
              break;
          }
        });
      }

      /**
       * Check if the views exposed form uses AJAX.
       *
       * @return {boolean|jQuery|*}
       *   true if Ajax is used, false otherwise.
       */
      function usesAjax() {
        return $('.views-filters-summary', context).hasClass(
          'views-filters-summary--use-ajax',
        );
      }

      /**
       * Returns the current view exposed form.
       *
       * @return {jQuery|*}
       *   The view exposed form.
       */
      function getExposedForm() {
        const $summaryElt = $('[data-exposed-form-id]', context);
        const exposedFormId = $summaryElt.data('exposed-form-id');
        return $(`form#${exposedFormId}`);
      }

      /**
       * Returns the exposed form filter button.
       *
       * @param {jQuery} $exposedForm
       *   The exposed form jQuery object.
       * @return {jQuery|*}
       *   The exposed form filter button.
       */
      function getFilterSubmit($exposedForm) {
        const $submit = $('input[type="submit"]:nth-child(1)', $exposedForm);
        // Some exposed forms can have multiple "filter" buttons.
        return $submit.first();
      }

      /**
       * Execute exposed form Ajax submit.
       */
      function ajaxSubmit() {
        const $exposedForm = getExposedForm();
        const $submit = getFilterSubmit($exposedForm);
        $submit.trigger('click');
      }

      /**
       * Click the views exposed form submit button or reload the page if
       * filters existed in the URL.
       *
       * @param {MouseEvent} event
       *   The click event.
       */
      function onRemoveClick(event) {
        event.preventDefault();
        const [selector, value] = $(this).data('removeSelector').split(':');
        if (usesAjax()) {
          const form = getExposedForm().get(0);
          const input = form.querySelector(`[name^="${selector}"]`);
          if (input !== null) {
            const $input = $(input);
            if (input.matches('input')) {
              switch ($input.attr('type')) {
                case 'radio':
                case 'checkbox':
                  $input.filter(`[value="${value}"]`).prop('checked', false);
                  break;
                default:
                  input.value = '';
                  break;
              }
            } else {
              $input.children(`[value="${value}"]`).prop('selected', false);
            }
          }
          ajaxSubmit();
        } else if (window.location.search.length > 0) {
          const url = new URL(window.location);
          // Remove the single value selector.
          url.searchParams.delete(selector);
          // Remove keyed elements used in grouped filters.
          url.searchParams.delete(`${selector}[${value}]`);
          // Remove the multi value selector, then add back in the ones that don't match the value.
          const multiValueParams = url.searchParams.getAll(`${selector}[]`);
          url.searchParams.delete(`${selector}[]`);
          multiValueParams.forEach((param) => {
            if (param !== value) {
              url.searchParams.append(`${selector}[]`, param);
            }
          });
          // window.location = url.toString();
          window.history.replaceState({}, document.title, url.toString());
          window.location.reload();
        } else {
          window.location.reload();
        }
      }

      /**
       * Reset all views exposed form filters.
       *
       * @param {MouseEvent} event
       *   The click event.
       */
      function onResetClick(event) {
        event.preventDefault();
        if (usesAjax()) {
          reset(getExposedForm());
          ajaxSubmit();
        } else {
          const uri = window.location.toString();
          const baseUrl = uri.substring(0, uri.indexOf('?'));
          window.history.pushState({}, '', baseUrl);
          window.location.reload();
        }
      }

      /**
       * Bind the remove and reset click events.
       */
      $('.views-filters-summary a.remove-filter', context).one(
        'click',
        onRemoveClick,
      );
      $('.views-filters-summary a.reset', context).one('click', onResetClick);
    },
  };
})(jQuery, Drupal, window);
